package mi.hdm.components;

import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.HBox;
import mi.hdm.recipes.RecipeComponent;

import static mi.hdm.helpers.Validation.isInteger;

public class SelectedIngredientLabel extends HBox {
    private final RecipeComponent component;
    private final TextField amountTextField;

    public SelectedIngredientLabel(RecipeComponent component) {
        super();
        this.component = component;
        amountTextField = new TextField();
        amountTextField.setPromptText("Enter amount");
        amountTextField.setOnAction(e -> validateAmountInput());
        amountTextField.setPrefWidth(75.0);
        Label componentNameLabel = new Label(component.getName());

        getChildren().addAll(amountTextField, componentNameLabel);

        setAlignment(Pos.BASELINE_LEFT);
        setSpacing(5.0);
        setStyle("-fx-padding: 5px");
    }

    private boolean validateAmountInput() {
        String text = amountTextField.getText().trim();
        if (!isInteger(text)) {
            amountTextField.clear();
            return false;
        }
        return true;
    }

    public Integer getAmount() {
        if (validateAmountInput()) {
            return Integer.parseInt(amountTextField.getText());
        }
        return null;
    }

    public RecipeComponent getComponent() {
        return component;
    }

    @Override
    public boolean equals(Object o) {
        if (o == this) return true;
        if (o instanceof SelectedIngredientLabel s) {
            return s.getComponent().equals(this.component);
        }
        return false;
    }
}
