package mi.hdm.controllers;

import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TextField;
import javafx.scene.layout.AnchorPane;
import mi.hdm.exceptions.InvalidIngredientException;
import mi.hdm.helpers.Validation;
import mi.hdm.recipes.Ingredient;
import mi.hdm.recipes.IngredientManager;
import mi.hdm.recipes.Measurement;
import mi.hdm.recipes.NutritionTable;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class IngredientCreatorController extends BaseController {
    private final static Logger log = LogManager.getLogger(IngredientCreatorController.class);

    private final IngredientManager ingredientManager;

    @FXML
    private AnchorPane parent;
    @FXML
    private TextField ingredientNameField;
    @FXML
    private ChoiceBox<Measurement> unitChoiceBox;
    @FXML
    private TextField caloriesTextField;
    @FXML
    private TextField carbsTextField;
    @FXML
    private TextField fatsTextField;
    @FXML
    private TextField fibersTextField;
    @FXML
    private TextField proteinsTextField;
    @FXML
    private TextField saltTextField;
    private TextField[] nutritionFields;

    public IngredientCreatorController(IngredientManager ingredientManager) {
        this.ingredientManager = ingredientManager;
    }


    //TODO: content is to high and overlapped by the header (also on Main page and some others)
    @FXML
    public void initialize() {
        loadHeader(parent);
        mapChoiceBox();
        nutritionFields = new TextField[]{caloriesTextField, carbsTextField, fatsTextField, proteinsTextField, fibersTextField, saltTextField};
    }

    @FXML
    public void submitCreateIngredient() {
        log.info("User submitted ingredient creation");
        try {
            Measurement selected = unitChoiceBox.getValue();
            Measurement submittedMeasurement = selected == null ? Measurement.GRAM : selected;
            String name = ingredientNameField.getText();

            Ingredient ingredient = new Ingredient(submittedMeasurement, name, new NutritionTable(
                    Double.parseDouble(caloriesTextField.getText()),
                    Double.parseDouble(carbsTextField.getText()),
                    Double.parseDouble(fatsTextField.getText()),
                    Double.parseDouble(proteinsTextField.getText()),
                    Double.parseDouble(fibersTextField.getText()),
                    Double.parseDouble(saltTextField.getText())
            ));
            ingredientManager.addIngredient(ingredient);
            log.info("Ingredient '{}' was created and added to ingredient manager", ingredient.getName());
            changeScene(View.MAIN);
        } catch (InvalidIngredientException e) {
            Alert a = new Alert(Alert.AlertType.ERROR);
            a.setHeaderText("Error creating ingredient");
            a.setContentText(e.getMessage());
            a.show();
            log.error("Ingredient was not created:");
            log.error(e.getMessage());
        } catch (NumberFormatException e) {
            Alert a = new Alert(Alert.AlertType.ERROR);
            a.setHeaderText("Error creating ingredient");
            a.setContentText("One or more of your nutrition values are invalid. Make sure to enter it like this: '10.2'");
            a.show();
            log.error("Ingredient was not created:");
            log.error(e.getMessage());
        }
    }

    private void mapChoiceBox() {
        final Measurement[] values = Measurement.values();
        for (Measurement m : values) {
            unitChoiceBox.getItems().add(m);
        }
    }

    /**
     * @return true if the name field is not empty and all the nutrition fields contain a string that can be parsed
     * to a double
     */
    private boolean inputsAreValid() {
        if (ingredientNameField.getText().isBlank()) return false;
        for (TextField f : nutritionFields) {
            String content = f.getText();
            if (!Validation.isDouble(content)) return false;
        }
        return true;
    }
}
