package mi.hdm.controllers;

import javafx.fxml.FXML;
import javafx.geometry.HPos;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.TextField;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import mi.hdm.components.IngredientSearchResultLabel;
import mi.hdm.mealPlan.MealPlan;
import mi.hdm.recipes.*;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

public class MealPlanController extends BaseController {
    private final MealPlan mealPlan;
    private final RecipeManager recipeManager;
    //private final IngredientManager ingredientManager;
    private final RecipeSearch recipeSearch;
    private List<Recipe> searchResults;

    private final static Logger log = LogManager.getLogger(MealPlanController.class);

    @FXML
    private AnchorPane parent;
    @FXML
    private StackPane stackPane;
    @FXML
    private GridPane mealPlanGrid;
    @FXML
    private ScrollPane searchScrollPane;
    @FXML
    private VBox searchVBox;

    public MealPlanController(MealPlan mealPlan, RecipeManager recipeManager, IngredientManager ingredientManager) {
        this.mealPlan = mealPlan;
        this.recipeManager = recipeManager;
        //this.ingredientManager = ingredientManager;
        recipeSearch = new RecipeSearch(recipeManager, ingredientManager);
        searchResults = new ArrayList<>();
    }

    @FXML
    public void initialize() {
        loadHeader(parent);
        displayMealPlan();
    }

    private void displayMealPlan() {
        mealPlanGrid.setGridLinesVisible(false);
        mealPlanGrid.getChildren().clear();
        mealPlanGrid.setGridLinesVisible(true);
        Font font = Font.font("System", FontWeight.BOLD, 17);
        Label labelToday = new Label("Today");
        labelToday.setFont(font);
        mealPlanGrid.add(labelToday, 0, 0);
        GridPane.setHalignment(labelToday, HPos.CENTER);
        for (int j = 1; j < 7; j++) {
            Label date = new Label(LocalDate.now().plusDays(j).format(DateTimeFormatter.ofPattern("dd.MM.")));
            date.setFont(font);
            mealPlanGrid.add(date, j, 0);
            GridPane.setHalignment(date, HPos.CENTER);
        }

        for (int j = 0; j < 7; j++) {
            final int J = j;
            mealPlan.getRecipeCodeForDay(LocalDate.now().plusDays(j)).ifPresentOrElse(code -> {
                Recipe r = recipeManager.getRecipeByCode(code);
                VBox displayRecipeVBox = new VBox();
                ImageView recipeImage = new ImageView(new Image(r.getImageURL().toString()));

                Label name = new Label(r.getName());
                name.setFont(font);
                displayRecipeVBox.getChildren().addAll(recipeImage, name);
                mealPlanGrid.add(displayRecipeVBox, J, 1);
                recipeImage.setPreserveRatio(true);
                recipeImage.fitWidthProperty().bind(mealPlanGrid.widthProperty().divide(7));
                recipeImage.fitHeightProperty().bind(mealPlanGrid.heightProperty().subtract(60));
                name.setPadding(new Insets(10));
            }, () -> {
                Button addRecipeButton = new Button("add recipe");
                mealPlanGrid.add(addRecipeButton, J, 1);
                addRecipeButton.setOnMouseClicked(element -> searchRecipes(LocalDate.now().plusDays(J)));
                GridPane.setHalignment(addRecipeButton, HPos.CENTER);
            });


        }
    }

       // @FXML
        public void searchRecipes(LocalDate date) {
            searchVBox = new VBox();
            searchVBox.setMaxSize(800, 800);
            searchVBox.setAlignment(Pos.CENTER);
            TextField searchTextField = new TextField();
            searchTextField.textProperty().addListener(e -> {
                        String input = searchTextField.getText();
                        log.debug("Input in search field changed, searching through recipes with query '{}'.", input);
                        searchResults = recipeSearch.searchByQuery(recipeManager.getRecipes(), input);
                        drawRecipeSearchResults(date);
                    }
            );
            searchScrollPane = new ScrollPane();
            searchTextField.setStyle("-fx-border-color: d91c1c; -fx-border-width: 4; -fx-border-radius: 10");
            searchScrollPane.setStyle("-fx-border-color: d91c1c; -fx-border-width: 4; -fx-border-radius: 10");
            searchVBox.getChildren().addAll(searchTextField, searchScrollPane);
            stackPane.getChildren().add(searchVBox);
        }

        private void drawRecipeSearchResults(LocalDate date) {
            log.debug("Drawing search results");
            VBox resultContainer = new VBox();
            searchResults
                    .forEach(
                            result -> {
                                IngredientSearchResultLabel resultLabel = new IngredientSearchResultLabel(result);
                                resultLabel.setOnMouseClicked(e -> {
                                    log.debug("User added recipe '{}' to mealplan for day {}.", result.getName(), date);
                                    mealPlan.addRecipeToMealPlan(result, date);

                                    stackPane.getChildren().remove(searchVBox);
                                    displayMealPlan();
                                    });
                                resultContainer.getChildren().add(resultLabel);
                            }
                        );
            searchScrollPane.setContent(resultContainer);
    }

    //TODO: displays nutrition score of the mealplan
}
