package mi.hdm.controllers;

import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import mi.hdm.GUIDriver;
import mi.hdm.TastyPages;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;

public enum ViewComponent {
    HEADER("/fxml/components/header.fxml");

    private static final Logger log = LogManager.getLogger(ViewComponent.class);

    private final String path;

    private static final TastyPages app = GUIDriver.getModel();

    ViewComponent(String path) {
        this.path = path;
    }

    public Parent getParentElement() throws IOException {
        log.info("Loading FXML for '{}' component from: {}", this, path);
        final FXMLLoader loader = new FXMLLoader(getClass().getResource(path));
        log.debug("Successfully retrieved resource from {}", path);

        switch (this) {
            case HEADER -> loader.setControllerFactory((callback) ->
                    new HeaderController(app.getRecipeManager(), app.getIngredientManager())
            );
        }

        log.debug("Set controller factory for view component {}", this);

        return loader.load();
    }
}
